/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
 /**
 * @file document.js
 * @brief Contains a Document class
 * @author Michiel Westerink
 */

const logging = require('./logging');
const utils = require('./utils');

module.exports = IMSDoc;

/**
 * Constructor for a Document object.
 *
 * @param string protonLink - Link as received from the web browser.
 */
function IMSDoc(protonLink)
{
    logging.logVerbose('IMSDoc: protonLink: ' + protonLink);
    //Split the link received from OpenIMS. Link contains the following data:
    //  0.  Protocol indicator  (proton:// for this application)
    //  1.  Version
    //  2.  Edit/view. If this field is 'view', it should be opened read-only
    //  3.  Hostname (IP address of hostname of server, depends on setting)
    //  4.  Sessionkey
    //  5.  Application to open file with. Not used in this application.
    //  6.  Includesubs - Not sure what this does
    //  7.  IMS Path (/site_collection/preview/objects/{32char string}/
    //  8.  Filename (document.docx for example)
    //  9.  GZ (Not used)
    //  10. HTTPS
    //  11. End
    var splittedLink = protonLink.split("%20");

//    logging.logVerbose('IMSDoc: stack: ' + utils.stacktrace());

    //HTTP(s) link to file.
    this.httpLink = splittedLink[10] + '://' + splittedLink[3] +
        splittedLink[7] + splittedLink[8];

    //If option equals 'view', the file is opened read-only
    this.readonly = splittedLink[2] === 'view' ? true : false;

    //The sessionkey
    this.sessionKey = splittedLink[4];

    //The 32 character string of the directory of the file
    //this.docDirectory = splittedLink[7].match(/[a-z0-9]{32}/);

    //ericd 011216 dir key is not necessarily a 32char string
    var docDirParts = splittedLink[7].split("/");
    this.docDirectory = docDirParts[4];

    //The hostname of the server from which the file has been downloaded.
    //Depending on the configuration on the server, this can either be an IP-
    //address or a hostname.
    this.hostName = splittedLink[3];

    //The name of the file (e.g. document.docx)
    this.fileName = splittedLink[8];
//    this.fileName = this.fileName.replace(/\[/g, '%5B'
   // 20170811 KVD SD08675 dirty hack must be solved in the whole chain of proton
    this.fileName = this.fileName.replace(/%(5B|5D)/g, function(m) {  return String.fromCharCode(parseInt(m.substring(1), 16)); });
    logging.logVerbose('IMSDoc: filename: ' + this.fileName);


    //The location where the file is stored for use on the client. By default
    //this is $USER_DIR/.proton_transferagent/{32 character string}/filename
    this.filePath = utils.getApplicationDirectory() + this.docDirectory + '/' +
        this.fileName;

    //The path to the file on the OpenIMS server
    this.imsSiteDirectory = splittedLink[7];

    //HTTPS yes/no
    this.https = splittedLink[10];

    //Empty by default, should be set by requesting an MD5 hash from the server
    //when the file has been downloaded or before it is uploaded.
    this.md5sum = '';

    this.plainText = false;
}

//ericd 011216 debugging
function ObjectToString (object)
{
    var s = "";
    for(var propertyName in object) {
        s = s + "[" + propertyName +"] => "+ object[propertyName]+"\r\n ";
    }
    return s;
}


function ShowDialog (string)
{
    var buttons = ['OK'];
    var dialogOptions = {
       'type' : 'info',
       'title' : 'dialog',
       'message' : string,
       'buttons' : buttons
    }
    dialog.showMessageBox(dialogOptions,
           function callback(buttonIndex) {
               if(buttonIndex === 0) {
                   //app.quit();
               }
           });
}
