/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
 /**
 * @file logging.js
 * @brief Provides function to write logs to files
 * @author Michiel Westerink
 */
const fs = require('fs');

const config = require('./config');
const utils = require('./utils');
var log = require('electron-log');

const logDisabledLevel = 0;
const logDebugLevel = 1;
const logVerboseLevel = 2;
const logInfoLevel = 3;
const logWarningLevel = 4;
const logErrorLevel = 5;

const util= require('util');

//var stackcounter = 0;  //  prevent nested debug logging


const logLevels = ["logDisabledLevel", "logDebugLevel", "logVerboseLevel", "logInfoLevel", "logWarningLevel", "logErrorLevel"];

/*
	available loglevels are
		export type ILogLevel = "error" | "warn" | "info" | "verbose" | "debug" | "silly";

*/

module.exports = {
    logToFile,
    logDebug,
    logVerbose,
    logInfo,
    logWarning,
    logError,
    clearLogFile
  }


/*
 * 20190826 GV clearLogFile function added  used to log set the logfile to zero bytes long
 */
 function clearLogFile() {

 	logDebug('clearLogFile');

 	logfile = config.getConfigItem('logFileLocation');

 	if (!logfile) {  // should be if it is empty !
 		logfile = config.getDefaultLogFileLocation();
 	}

	fs.writeFile(logfile, "", function (err) {
		if (err) {
			throw err;
			logError("ClearLogFile Error "+err);
		}
	});

}


/**
 * 20190711 GV six new log routines added. Used to log depending on the loglevel
 * @param string data - Data to log
 */
function checkloglevel(level) {
    try {
        thelevel = Number(config.getConfigItem('loggingEnabled'));
        if (thelevel <= level) {
            return true;
        }
    } catch (e) {
        thelevel = logDebugLevel;
        return true;
    }
    return false;
}

function logDebug(data, where) {
    logToFile(data,1,logDebugLevel);
}

function logVerbose(data, where) {
    logToFile(data,1,logVerboseLevel);
}

function logInfo(data, where) {
    logToFile(data,1,logInfoLevel);
}

function logWarning(data, where) {
    logToFile(data,1,logWarningLevel);
}

function logError(data, where) {
    logToFile(data,1,logErrorLevel);
}



/**
 * A SIMPLE way of logging without levels
 *
 * Logs to default logging location "log.transports.file.file"
 * data: the data to be logged
 * level : stack trace level (nonzero of called from another 'debugging' function)
 * loglevel: logging level
 * @return <nothing>
 */
function logToFile(data,level,loglevel)
{
    const loglevelsverbose = ["", "debug", "verbose", "info", "WARNING", "**ERROR**"];

    if (typeof(data) == "object") {
        data = util.inspect(data, {showHidden: false, depth: null});
    }
    if (!loglevel) {
        loglevel = 0;
    }
    if (typeof(logToFile.stackcounter) == "undefined") {
        logToFile.stackcounter = 0;
    }


    var currentloglevel = 1; // debugging
    currentloglevel = config.getConfigItem('loggingEnabled');  // CRASHES 'stack too large' in some cases !!!!!
    if (logToFile.stackcounter > 0) {
        log.info("stacktoolarge:"+logToFile.stackcounter + " " + data); return;
        return;
    }
    logToFile.stackcounter++;
    // do not log if set to lower log level, if not defined (yet, before initialization of config), log everything
    if (currentloglevel && Number(currentloglevel) > loglevel) {
        //log.info("currentloglevel="+currentloglevel + ",loglevel=" + loglevel + ",stackcounter=" + logToFile.stackcounter);
        logToFile.stackcounter--;
        return;
    }

    if (!level) {
        level = 0;
    }
    level += 3;
    thetrace = utils.stacktrace(level);
    thetrace = thetrace.replace(/:\d+\)/, ")");  // no column number
//    data = thetrace  +  data;  // the fourth line of the stack trace is where I'm called from
    data = "[" +loglevelsverbose[loglevel] + "]" + thetrace + data;
    data = data.replace(/(\r\n|\n|\r)/gm, "");
    log.info(data);   // 20170818 KVD use log library
    logToFile.stackcounter--;
}
/**
 * Create a timestamp that is used in the log. Format is as follows:
 * YYYY-MM-DD hh:mm:ss
 *
 * @return string - Timestamp in aforementioned format
 */
function date()
{
    var now = new Date();
    var year = now.getFullYear();

    var month = (now.getMonth()) + 1;
    month = month < 10 ? '0' + month : month;

    var date = now.getDate();
    date = date < 10 ? '0' + date : date;

    var hour = now.getHours();
    hour = hour < 10 ? '0' + hour : hour;

    var minute = now.getMinutes();
    minute = minute < 10 ? '0' + minute : minute;

    var seconds = now.getSeconds();
    seconds = seconds < 10 ? '0' + seconds : seconds;

    var timestamp = year + '-' + month + '-' + date + ' ';
    timestamp = timestamp + hour + ':' + minute + ':' + seconds;

    return timestamp;
}
