/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
 /**
 * @file main.js
 * @brief Initialisation of the application etc.
 * @author Michiel Westerink
 */

const args = require('yargs').parse(process.argv.slice(1))._;
const child_process = require('child_process');
const electron = require('electron');
const fs = require('fs');
const http = require('http');

const {BrowserWindow} = electron;
const {app} = electron;
const {dialog} = require('electron');
const {protocol} =  require('electron');

const config = require('./config');
const logging = require('./logging');
const openfile = require('./openfile');
const utils = require('./utils');
const settingsWindow = require('./settingsWindow.js');

const {globalShortcut} = require('electron')

var log = require('electron-log');
var documentopened = null;

// Write to this file, must be set before first logging
log.transports.file.level = true;
log.transports.console.level = true;
log.transports.file.file = config.getConfigItem("logFileLocation"); // utils.getApplicationDirectory()+ 'log.txt';
//!!
//log.transports.file.file = utils.getApplicationDirectory()+ 'log.txt';
log.transports.file.format = '{y}-{m}-{d} {h}:{i}:{s} {text}';

// https://stackoverflow.com/questions/30681639/how-to-access-browserwindow-javascript-global-from-main-process-in-electron#30747290
// In main process.
const ipcMain = require('electron').ipcMain;
// In renderer process (web page).
const ipcRenderer = require('electron').ipcRenderer;

//URL will be stored in this variable for later use
var paramURL = '';

let win;
let settingsWin

//Used on OSX (seen open-url event handler)
var protonChild = false;


//const remote = require('electron').remote;

const { remote } = require( "electron" );

//const { ipcRenderer, remote } = require( "electron" );

// https://electronjs.org/docs/all#how-to-share-data-between-web-pages
// In the main process.
global.storage = {
  setWin: null,
  lsofchildPID: null
}

/*
	JG this function create the waiting for window when opening a file on the server for example.
*/
function createWindow()
{
    logging.logDebug('createWindow: ');
    //Create tmp directory etc.
    init();

    var fs = require('fs')
    var json = JSON.parse(fs.readFileSync(`${__dirname}/package.json`, 'utf8'))

logging.logDebug("TMPSPLIT="+documentopened);
    if(protonChild || utils.detectPlatform() === 'linux') {
        //Set window-size here
        var debug = config.getConfigItem('debuggingEnabled');
    	var windowWidth = 400;
    	if ( debug == true ) windowWidth += 500;

    	var windowHeight = 100;
    	if ( debug == true ) windowHeight += 400;


        win = new BrowserWindow({width: windowWidth, height: windowHeight, title: 'FileTransferAgent ver. ' + json.version});
        win.setResizable(false);

        logging.logDebug("main.js createwindow", false);
        logging.logDebug('json='+utils.objectToString(json));
        // 20170804 KVD SD08674
        // https://stackoverflow.com/questions/44611317/using-console-log-with-electron
//        if(config.getConfigItem('debuggingEnabled') === true) {
        if(config.getConfigItem('debuggingEnabled') == 1) {
            win.openDevTools();
            win.setResizable(true);
        }

        var res = win.loadURL(`file://${__dirname}/index.html`);
        logging.logDebug('res: ' + res);

        win.on('closed', () => {
            win = null;
        });

        //On Linux, things are a bit different, the URL is passed as a param to
        //the executable, instead of via an event, such as on OSX
        if(utils.detectPlatform() === 'linux') {
            logging.logDebug('args: ' + args);

            var IMSDoc = require('./document');
            var imsDoc = new IMSDoc(args + '');

            openfile.verifyLink(imsDoc);
        }
        // 20210219 KVD SD10432 Open settings window anyway when no file is transferred
    } else if (!documentopened) {
        settingswindow();
    }
}
/**
 * This event will only be triggered on OSX, other platforms (Linux at least)
 * will get the URL as a parameter instead of via this event.
 *
 * Also, on Linux a new  instance of the application is opened everytime, so
 * the protonChild shenanigans can be left out.
 */
app.on('open-url', function(event, url)
{
    event.preventDefault();

    //Create tmp directory etc.
    init();

    var tmpSplit = url.split('%20');
    // tell whether a document is opened, hence do not open the settings window
    if (tmpSplit) {
        documentopened = tmpSplit;
    }

    //This should be better, but it works quite well for now. Explanation below:
    //
    //If the application is launched from the browser, an instance of the
    //application is opened, which appends 'child' to the end of the URL. After
    //this, the application launches a new instance of the application with
    //this new URL, closing the current one.
    //
    //This way, multiple instances of the TransferAgent can be opened at the
    //same time. There should be a way to have OSX always launch a new instance
    //of the application, but I am yet to find it.
    logging.logDebug('tmpsplit='+tmpSplit);
    if(tmpSplit[tmpSplit.length - 1] === 'child') {
        logging.logDebug('child=');
        paramURL = url;
        protonChild = true;
        var IMSDoc = require('./document');

        //The 'open-url' event usually fires before the 'ready' event. This causes
        //some nasty issues, so the URL is stored in paramURL, which is then used
        //when the 'ready' event has been fired.
        var imsDoc = new IMSDoc(paramURL);
        logging.logDebug('imsDoc=',imsDoc);

        //Sometimes the 'ready' event occurs a little too late, which causes
        //a problem when a MessageBox should be shown (this can only be done
        //after the 'ready' event has occured).
        setTimeout(function() {
            //Before doing anything, verify the URL
            openfile.verifyLink(imsDoc);
        }, 1000);
    } else {
        url = url + '%20child';

        //Force a new instance of the TransferAgent to be opened
        child_process.exec ('open -n ' + url, function (error, stdout, stderr) {
            if(!protonChild) {
                app.quit();
            }
        });
    }
});

app.on('quit', function(event) {
    event.preventDefault();
    logging.logDebug('Quit !');
});


// app.on('ready', createWindow );

/*
app.on('ready', () => {

  logging.logDebug('app.on(ready) !');


  createWindow();

});

*/

function settingswindow()
{
	logging.logDebug('menu Settings click => settingsWindow.createSettingsWindow() !');

	storage = global.storage;
	logging.logDebug("menu Settings click storage = \n"+utils.objectToString(storage));

	sw = storage.setWin;

	// objectToString(sw) is [object BrowserWindow] with lots of functions
	//logging.logDebug("menu Settings click sw = \n"+utils.objectToString(sw));

	setWin = global.storage.setWin;
	//logging.logDebug("menu Settings click setWin = \n"+utils.objectToString(setWin));


	settingsWindow.createSettingsWindow();


	storage = global.storage;
	logging.logDebug("menu Settings click storage = \n"+utils.objectToString(storage));

	sw = storage.setWin;
	//logging.logDebug("menu Settings click sw = \n"+utils.objectToString(sw));

	setWin = global.storage.setWin;
	//logging.logDebug("menu Settings click setWin = \n"+utils.objectToString(setWin));
}

app.on('ready', () => {

    createWindow(); // 20210205 KVD SD12265 Show main window
//    init();

    /*
    logging.logDebug('app.on ready logDebug');
    logging.logVerbose('app.on ready logVerbose');
    logging.logInfo('app.on ready logInfo');
    logging.logWarning('app.on ready logWarning');
    logging.logError('app.on ready logError');
    */
	//logging.logDebug('app.on ready !');
    var editmenu = "";
    logging.logDebug(config.getConfigItem('debuggingEnabled'));
    var menuparams = [
        {
            label: 'Electron',
            submenu: [
            {
                label: utils.ML('Voorkeuren','Preferences'),
                accelerator: 'Cmd+,',
                click: function() {
                    settingswindow();
                }
            },
            {
                label: utils.ML('Log bekijken','View log'),
                accelerator: 'Cmd+0',
                click: function() {
                	logging.logDebug('menu View log => config.getConfigItem("logFileLocation" !');

                	var logFileLocation = config.getConfigItem("logFileLocation");

                	logging.logDebug('menu View log => logFileLocation = '+logFileLocation);
                    electron.shell.openItem( logFileLocation );
                }
            },
            { type: 'separator' },
            {
                label:
                utils.ML("Over","About")+ " Transfer Agent", selector: "orderFrontStandardAboutPanel:"
            },


            { type: 'separator' },
            {
                label: utils.ML('Afsluiten','Quit'),
                accelerator: 'Cmd+Q',
                click: function() {
                	logging.logDebug('menu Quit => app.quit() !');
                    app.quit()// JG generated an error here
                    //app.send('window-all-closed')
                }
            }
            ]
        }
    ];
    // 20210402 KVD SD12255 When debugging enabled, allow copy/paste as copy/paste in console does NOT work in Electron !
    // https://pracucci.com/atom-electron-enable-copy-and-paste.html
    if (config.getConfigItem('debuggingEnabled') != 0) {
        logging.logDebug("DEBUG ON !");
        editmenu =
        {
            label: "Edit/copy/paste (debug)" ,
            submenu: [
                { label: "Undo", accelerator: "CmdOrCtrl+Z", selector: "undo:" },
                { label: "Redo", accelerator: "Shift+CmdOrCtrl+Z", selector: "redo:" },
                { type: "separator" },
                { label: "Cut", accelerator: "CmdOrCtrl+X", selector: "cut:" },
                { label: "Copy", accelerator: "CmdOrCtrl+C", selector: "copy:" },
                { label: "Paste", accelerator: "CmdOrCtrl+V", selector: "paste:" },
                { label: "Select All", accelerator: "CmdOrCtrl+A", selector: "selectAll:" }
            ]
        };
        menuparams[0].submenu.splice(2,0,editmenu);  // after menuitem 2 (3rd 1-based)
    }

    const menu = electron.Menu.buildFromTemplate(menuparams);
    electron.Menu.setApplicationMenu(menu);

    //if (win === null) {
    //createSettingsWindow();
    //}
});

/*
app.on('activate', () => {
    //if (win === null) {
    //    createSettingsWindow();
    //}

});
*/

app.on('window-all-closed', () => {
    //On macOS it is common for applications and their menu bar
    //to stay active until the user quits explicitly with Cmd + Q
    if (process.platform === 'darwin') {
        app.quit(); // 20210212 KVD Enabled: app should exit !
    }
});

/**
 * Check if configuration files, application directories etc. are present. If
 * they are not, create them.
 * gv 29-04-2019
 * if (configFilePath) settings.json is deleted recreate it
 */
function init()
{
    try {
        fs.statSync(utils.getApplicationDirectory());
    } catch (e) {
        if(e.toString().indexOf("ENOENT") !== -1) {
            fs.mkdirSync(utils.getApplicationDirectory());
            config.initConfig();
        }
    }
	configFilePath = utils.getApplicationDirectory() + 'settings.json';
	try {
	//	logging.logVerbose("=============== configFilePath "+configFilePath);
	  if (fs.existsSync(configFilePath)) {
		//file exists
//		logging.logVerbose("=============== configFilePath file exists "+configFilePath);
	  } else {
	  	// recreate configFilePath
	  	config.initConfig();
	  }
	} catch(err) {
		// recreate configFilePath
	  	config.initConfig();
	}
    //Self-signed certificates should always be accepted
    process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
}
