/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
/**
 * @file settingsWindow.js
 * @brief
 * @author Johnny Groeneveld
 */
//const fs = require('fs');
//const config = require('./config');
//const utils = require('./utils');
//var log = require('electron-log');

const logging = require('./logging');

const electron = require('electron');
const {BrowserWindow} = electron;
const {ipcMain} = require('electron')
const {writeConfigToFile} = require('./config')
const {readConfigFile} = require('./config')

const remote = require('electron').remote;

const config = require('./config');

module.exports = {
    createSettingsWindow
}

function createSettingsWindow()
{
	setWin = global.storage.setWin;

	logging.logDebug("createwindow setWin = "+ setWin , false);
	if (setWin) {
		logging.logDebug('createSettingsWindow got setWin !'+setWin);
		setWin.show();
		return;
	}
    var fs = require('fs')
    var json = JSON.parse(fs.readFileSync(`${__dirname}/package.json`, 'utf8'))

    var debug = config.getConfigItem('debuggingEnabled');
	//var debug = false;// JG - Set to true to enable chrome debugger
	//logging.logDebug('createSettingsWindow getConfigItem debuggingEnabled !'+debug);

	var windowWidth = 600;
	if ( debug == true ) windowWidth += 500;

	var windowHeight = 500;
	if ( debug == true ) windowHeight += 300;

    settingsWin = new BrowserWindow({width: windowWidth, height: windowHeight, title: 'FileTransferAgent ver. ' + json.version});
    settingsWin.setResizable(false);

	if ( debug == true) {
		logging.logDebug('createSettingsWindow debuggingEnabled == true use openDevTools!!!!!'+debug);
    	settingsWin.openDevTools();
	}
    settingsWin.loadURL(`file://${__dirname}/settingsWindow.html`);
	ipcMain.on('clearlogFile-submit-evt', (event, formData) => {
		logging.logDebug('clearlogFile-submit-evt received!!! '); // logDebug is quite useless if we going to clear the logfile but maybe that's what we want to debug...
		logging.clearLogFile();
  	});

	ipcMain.on('settings-submit-evt', (event, formData) => {

		logging.logDebug("settings-submit-evt start ");

		var oldconfig = readConfigFile();

		 for (var key in oldconfig) {
			logging.logDebug("oldconfig[" + key + "]" + " = " + oldconfig[key]);
		}

 		// send message to index.html
 		var configuration = new Object;
 		var lsofExtensions = new Array;
        logging.logDebug(formData);
 		for ( var i=0;i!=formData.length;i++ ) {
 			logging.logDebug(formData[i].name + " ("+i+") " + formData[i].value + " typeof = "  + typeof(formData[i].value) );

 			if (formData[i].name == "lsofExtension") {
 				lsofExtensions.push(formData[i].value);
 			} else {
 				configuration[ formData[i].name ] = formData[i].value;
 			}
 		}
 		if (!configuration.forceLockEnabled) {
 			configuration.forceLockEnabled = false;
 		}
 		logging.logDebug("lsofExtensions.length =  " +lsofExtensions.length );
 		logging.logDebug(lsofExtensions );

 		configuration[ 'lsofExtensions' ] = lsofExtensions;

 		for (var key in configuration) {
			logging.logDebug("configuration[" + key + "]" + " = " + configuration[key]);
		}
        logging.logDebug(configuration);
 		writeConfigToFile(configuration);
		event.sender.send('settings-submit-reply', 'closeWindow' );
  	});

    settingsWin.on('closed', () => {
        settingsWin = null;
        global.storage.setWin = null;
    });
    global.storage.setWin = settingsWin;
}
