/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
 /**
 * @file settingsWindowRenderer.js
 * @brief
 * @author Johnny Groeneveld
 */

//alert(1);
//var ipcRenderer = require(‘electron’).ipcRenderer;

const utils = require('./utils');
const {ipcRenderer} = require('electron');
const config = require('./config');
const logging = require('./logging');

/*
var console = {
    log: function(text) {
         logging.logToFile(text,1,logDebugLevel);
    }
};
*/
//ipcRenderer.send('hello','a string', 10);

function loadCurrentConfigurationToForm() {
    //JG DEZE CODE MOET NAAR CONFIG VIND IK.
    //const appdir = utils.getApplicationDirectory()
    //const configuration = require( appdir + "settings.json" )
    const configuration = config.readConfigFile();
    loadConfigurationToForm( configuration );
}

function loadDefaultConfigurationToForm() {
    const defaultConfiguration = config.readDefaultConfig();
    loadConfigurationToForm( defaultConfiguration );
}

// 20210402 KVD Handler when 'use local file' is enabled
function disabletakeoverlock(disabled)
{
    if (disabled) {
        $("#forceLockEnabledrow").hide();
    } else {
        $("#forceLockEnabledrow").show();
    }
    //$("#forceLockEnabled").prop("disabled",disabled);
    console.log(disabled);
}

function loadConfigurationToForm( configuration ) {
    var keys = Object.keys(configuration);
//    logging.logDebug(keys);
    // 20210219 KVD SD10432: Made menu buttons dynamical
    var labels = {
        "language"                : utils.ML("Taal", "Language"),
        "logFileLocation"         : utils.ML("Logbestand locatie", "Log file location"),
        "waitForApplication"      : utils.ML("Wachttijd applicatie", "Wait time application"),
        "forceLockEnabled"        : utils.ML("Neem lock niet over","Do not take over lock"),
        "alwaysuselocalfile"      : utils.ML("Gebruik lokaal bestand","Use local file"),
        "maxFileSizeInMegabytes"  : utils.ML("Maximum upload", "Maximum upload"),
        "seconds"                 : utils.ML("seconden", "seconds"),
        "lsofextensies"           : utils.ML("De volgende bestandstypes worden automatisch in het DMS gezet:",
                                             "The following exiensions will be automatically transferred to DMS:")
    };
  //  logging.logDebug(labels);
    for (var id in labels) {
  //      logging.logDebug(id);
        $("label[for='"+id+"']").text(labels[id]);
    }

    var dropdowns = {
        "loggingEnabled" : {
            "1" : utils.ML("Ontwikkelaar", "Debug"),
            "2" : utils.ML("Uitgebreid", "Verbose"),
        //    "3" : utils.ML("Informatief", "Info"), // 20210402 KVD SD12255
            "4" : utils.ML("Waarschuwing", "Warning"),
            "5" : utils.ML("Fouten", "Error"),
            "10": utils.ML("Geen", "None")
        },
        "debuggingEnabled" : {
            "1" : utils.ML("Aan", "On"),
            "0" : utils.ML("Uit", "Off")
        },
        "forceLockEnabled" : {
            "1" : utils.ML("Aan", "On"),
            "0" : utils.ML("Uit", "Off")
        },
        "alwaysuselocalfile" : {
            "1" : utils.ML("Aan", "On"),
            "0" : utils.ML("Uit", "Off")
        },
        "language" : {
            "nl" : "Nederlands",
            "en" : "English"
        }
    };
    logging.logDebug(dropdowns);

    for (var i in dropdowns) {
        for (var j in dropdowns[i]) {
            $("#" + i).append(new Option(dropdowns[i][j], j));
        }
    }

    var buttons = {
        "submit"                  : utils.ML("Opslaan", "Save"),
        "close-btn"               : utils.ML("Sluiten", "Close"),
        "clearlogFile"            : utils.ML("Logbestand wissen", "Clear log file"),
        "lsof-add-ext-btn"        : utils.ML("Bestandstype toevoegen", "Add file type"),
        "selectfolder"            : utils.ML("Kies log bestand folder", "Select log file folder"),
        "defaultSettings"         : utils.ML("Standaard instellingen", "Default settings")

    };

    for (var i in buttons) {
        $("#"+i).html(buttons[i]);
    }

    for (var i=0;i < keys.length; i++ ) {
        var key = keys[i];

        if (key == "lsofExtensions") {
            ClearAllLsofItems();

            // lsofExtensions should be added dynamicly
            lsofExtensions = configuration[ key ];
            console.log( key  + " : " + configuration[ key ]);

            for (var j=0;j < lsofExtensions.length; j++ ) {
                addlsofItem(lsofExtensions[ j ]);
            }
            //addlsofItem("");// JG add an empty to be able to set a new
        } else {
            var domFormElement = document.getElementById(key);
            if ( domFormElement ) {
                domFormElement.value = configuration[ key ];
            }
        }
    }
    // 20210402 KVD sd12255 Use local file
    $("#alwaysuselocalfile").bind("change", function ()
    {
        let disabled = ($(this).val() === "1");
        disabletakeoverlock(disabled);
    });
    if (config.getConfigItem('alwaysuselocalfile') == 1) {
        disabletakeoverlock(true);
    }
    // TEST only
    if (config.getConfigItem('debuggingEnabled') == 1) {
        $(".debugonly").show();

        $("#selectfolder").click(function() {
            var path = selectlogfilefolder();
            if (path) {
                logging.logDebug(path);
                $("#logFileLocation").val(path);  // update accompanying text field
                logging.logDebug($("#logFileLocation").val());
                // value is being saved in ipcMain.on('settings-submit-evt', (event, formData)
                // but when click on 'Kies', the window closes and reopens and thus restoring the old value
                // so save here anyway
                config.changeConfig('logFileLocation', path);
            }
        });
    }
}


// https://www.electronjs.org/docs/api/dialog
function selectlogfilefolder()
{
    // https://www.brainbell.com/javascript/show-save-dialog.html
    const {remote} = require('electron');
    let dialog = remote.dialog;
    let path = "";
    let curwin = remote.getCurrentWindow();

    const options = {   title: utils.ML("Kies folder", "Select folder"),
                        nameFieldLabel: utils.ML("Bestandsnaam", "Filename"),
                        showsTagField: false,
                        buttonLabel: utils.ML("Kies", "Select"),
                        defaultPath: config.getConfigItem("logFileLocation")
                    };
    try {
        path = dialog.showSaveDialog(curwin, options);
        // when dialog canceled, it returns 'undefined'
        if (typeof path == "undefined" || !path) {
            path = false;
        }
    } catch (e) {
        logging.logError(e);
    }
    return path;
}

$(document).ready( function() { loadCurrentConfigurationToForm(); } );


document.querySelector('#clearlogFile').addEventListener('click', function() {
    if (confirm("Weet u zeker dat u alle logging wilt verwijderen?")) {
        ipcRenderer.send('clearlogFile-submit-evt', $('form').serializeArray() );
    }
});

$('#defaultSettings').bind("click" , function() {
    defaultSettingsButtonClicked();
}
);

function removeItem(what) {
    $(what).parent('.lsofItem').remove();
}

function addlsofItem( value )
{
    var newlsofinput = $('<input class="lsofExtension" required="true" placeholder="Extension" type="text" name="lsofExtension" size="5" maxlength="4" value="'+value+'" >');
    var newlsofremovebutton = $('<button type="button" tabindex="-1" class="removelsofExtension">X</button>');

    $("#lsofextensies").append($('<div class="lsofItem">').append( newlsofinput ).append( newlsofremovebutton )) ;

    $(newlsofremovebutton).bind("click" , function() { removeItem(this);} );
    //$(newlsofinput).bind("change" , function() { if ( this.value=="") removeItem(this);} );

    if ( !value ) {
        newlsofinput.focus();
    }
}

function ClearAllLsofItems() {
    $("#lsofextensies").html("");
}

function defaultSettingsButtonClicked() {
    //alert("Standaard instellingen worden teruggezet.\nKlik op Opslaan op ze te bewaren."); return false;
    if ( confirm(utils.ML("Weet U zeker dat u de standaard instellingen wilt terugzetten?",
                          "Are you sure you want to reset the default settings ?" ) ) ) {
        loadDefaultConfigurationToForm();
    }
    return false;
}

document.querySelector('#lsof-add-ext-btn').addEventListener('click', function() {
    return addlsofItem("");
});

document.querySelector('#submit').addEventListener('click', function() {
    // send submit to main.js

    /*
    initial names: asynchronous-message and asynchronous-reply
    ipcRenderer.send('asynchronous-message', $('form').serializeArray() );
    ipcRenderer.on('asynchronous-reply', (event, action) => {
      console.log(action)
      if ( action=='closeWindow' ) {
     //     window.close();
      }
    });
    */
    ipcRenderer.send('settings-submit-evt',  $("form").serializeArray() );

    // receive message from main.js
    ipcRenderer.on('settings-submit-reply', (event, action) => {
      if ( action=='closeWindow' ) {
          window.close();
      }
    });

});

document.getElementById("close-btn").addEventListener("click", function (e) {
    window.close();
});

//document.getElementById("waitForApplication").value = configuration[3].waitForApplication;¬
