/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
/**
 * @file utils.js
 * @brief Provides various generic functions that are commonly used throughout
 * the application, such as detecting the platform etc.
 * @author Michiel Westerink
 */

module.exports = {
    detectPlatform,
    getApplicationDirectory,
    getDocumentMD5,
    compareLocalWithRemote,
    cleanUp,
    markCacheFile,
    isPlainTextFile,
    hasKnownLsofExt,
    waitForMessageBox,
    objectToString,
    showDialog,
    stacktrace,
    ML,
    newCreateWindow
}

const {app} = require('electron');
const {dialog} = require('electron');

const child_process = require('child_process');
const fs = require('fs');
const md5File = require('md5-file');
const request = require('request');

const config = require('./config');

const logging = require('./logging');

var log = require('electron-log');

const util = require('util');




/*
	JG this function create the waiting for window when opening a file on the server for example.
*/
function newCreateWindow()
{



logging.logDebug('newCreateWindow: ');


return;

    //Create tmp directory etc.
//    init();

    var fs = require('fs')
    var json = JSON.parse(fs.readFileSync(`${__dirname}/package.json`, 'utf8'))

    if(protonChild || utils.detectPlatform() === 'linux') {
        //Set window-size here
        win = new BrowserWindow({width: 400, height: 100, title: 'FileTransferAgent ver. ' + json.version});
        win.setResizable(false);

        logging.logDebug("main.js createwindow", false);
        logging.logDebug('json='+utils.objectToString(json));
        // 20170804 KVD SD08674
        // https://stackoverflow.com/questions/44611317/using-console-log-with-electron
        if(config.getConfigItem('debuggingEnabled') === true) {
            win.openDevTools();
            win.setResizable(true);
        }

        win.loadURL(`file://${__dirname}/index.html`);

        win.on('closed', () => {
            win = null;
        });

        //On Linux, things are a bit different, the URL is passed as a param to
        //the executable, instead of via an event, such as on OSX
        if(utils.detectPlatform() === 'linux') {
            logging.logDebug('args: ' + args);

            var IMSDoc = require('./document');
            var imsDoc = new IMSDoc(args + '');

            openfile.verifyLink(imsDoc);
        }
    }
}



/**
 * Detect the platform the application runs on. Values for the supported
 * platforms are:
 *  - darwin for macOS
 *  - linux for Linux based operating systems (Ubuntu, Mate, Fedora etc.)
 *  - win32 for Windows
 *
 *  @return string - String indication the platform (see above)
 */
function detectPlatform()
{
    return process.platform;
}

/**
 * Get the application directory in which configuration files, temporary files
 * etc. can be stored.
 *
 * Note that a '/' or a '\' (depending on the platform) is appended after the
 * returned path. This is eliminates the need to check which platform is used
 * when using this function.
 *
 * By default, the appliction directories are as follows:
 *  - Mac/Linux: $HOME/.proton_transferagent
 *  - Windows: %USERPROFILE%\.proton_transferagent
 *
 * @return string - Path to the application directory
 */
function getApplicationDirectory()
{
    //ericd 090117 changed dirname (meant for OSX)
    switch(detectPlatform()) {
        //Linux and OSX can be handled using the same code, special exception
        //made for win32 because it's a 'special' OS
        case "darwin": //Fall through to 'linux' case, because code is the same
        case "linux":
            return process.env['HOME'] + '/OpenImsTransferAgent/';
            break;
        case "win32":
            return process.env['USERPROFILE'] + '\\OpenImsTransferAgent\\';
            break;
        default:
            break;
    }
}

/**
 * Set the MD5 hash of a document that is present on disk.
 *
 * @param IMSDoc imsDocument - IMSDocument to verify
 */
function getDocumentMD5(imsDocument)
{
    //Compose the path to the downloaded file
    filePath = getApplicationDirectory() + imsDocument.docDirectory + "/";
    filePath += imsDocument.fileName;

    try {
        //Calculate MD5 sum and compare with downloaded MD5 sum
        imsDocument.md5sum = md5File.sync(filePath);
    } catch(e) {
        return;
    }
}

/**
 * See if the local file and the remote file differ by comparing MD5 hashes.
 *
 * @param IMSDoc imsDocument - Document to compare
 */
function compareLocalWithRemote(callbackFunction, imsDocument)
{
//    logging.logVerbose('compareLocalWithRemote: imsDocument: ' + utils.objectToString(imsDocument));
//    logging.logVerbose('compareLocalWithRemote: stacktrace: ' + utils.stacktrace());


    var formData = {
        'command' : 'openims_get_blockmd5',
        'file' : imsDocument.imsSiteDirectory + imsDocument.fileName,
        'offset' : 0,
        'size' : getDocumentSize(imsDocument)
    };

    request.post({url: imsDocument.https + '://' + imsDocument.hostName +
        '/nkit/gate.php',
        formData : formData}, function callback(err, response, body) {
            if(err) {
                var message = '';
                if(JSON.stringify(err).indexOf('SELF_SIGNED_CERT') !== -1) {
                    message = ML('Certificaatfout', 'Certificate error');
                } else {
                    message = ML('Onbekende fout', 'Unknown error');
                }
                dialog.showErrorBox(ML('Fout', 'Error'), message);
                app.quit();
            }

            getDocumentMD5(imsDocument);

            var different = false;
            if(body === imsDocument.md5sum) {
                different = false;
            } else if (body !== imsDocument.md5sum) {
                different = true;
            }

            callbackFunction(imsDocument, different);
        });
}

/**
 * Clean up after the user is done editing the document. This is done by
 * removing the temporary folder in which the document has been saved. This
 * folder is usually located in the following directory:
 *  $USER_DIR/.proton/{imsDirectoryOfFile}
 *
 * @param IMSDoc imsDocument - IMSDocument to clean up for
 */
function cleanUp(imsDocument)
{
    var cachePath = getApplicationDirectory() + imsDocument.docDirectory +
        '/cache';

    try {
        fs.unlinkSync(cachePath);
        app.quit();
    } catch(e) {
        app.quit();
    }
}

/**
 * Get the size (in Bytes) of a document
 *
 * @param IMSDoc imsDocument - Document to get size from
 * @return int - Size of file in Bytes
 */
function getDocumentSize(imsDocument)
{
    var fileStats = fs.statSync(imsDocument.filePath);

    return fileStats.size;
}

/**
 * Create a file in the directory of a downloaded document to indicate whether
 * or not it has been uploaded. If the TransferAgent detects that the file has
 * been downloaded already, but there are still local changes that haven't been
 * uploaded to the server, the user is prompted if the TransferAgent should
 * open the remote or the local file.
 *
 * @param IMSDoc imsDocument - Document to mark
 */
function markCacheFile(imsDocument)
{
    var path = getApplicationDirectory() + imsDocument.docDirectory +
        '/cache';

    var timestamp = new Date();

    fs.writeFileSync(path, timestamp, {flag: 'w'});
}

/**
 * Gets the filetype from a given file. This can be used to determine which
 * application should be opened.
 *
 * @param string fileName - Path of the file to determine the filetype of
 * @return string - Extension of the file
 */
function getFileType(fileName)
{
    //Regex gets the file extension of a file by extracting the last
    //5 characters between a '.' (dot) and EOL character
    return fileName.match(/\.[a-zA-Z]{1,5}$/)[0];
}

/**
 * When a document has been opened, the TransferAgent uses the 'lsof' utility
 * to determine whether or not a process still has the document in use. This
 * works fine for .docx, .xlsx, .pptx etc files (and presumably all binary
 * files), but not for plaintext files.
 *
 * For some reason, the process that owns these files is the 'filecoord'
 * daemon, which runs under the root user. When executing the 'lsof' command as
 * a non-privileged user, the file doesn't show up in the output and thus, the
 * TransferAgent assumes it is no longer in use, so it can shut down.
 *
 * Because of this, the TransferAgent needs another way to determine if the
 * user is done editing the documenet. As of now, the only way to only way to
 * determine this is to let the user signal this to the TransferAgent by
 * clicking a messagebox.
 *
 * This function works for both OSX and Linux, not for Windows.
 *
 * @param IMSDocument imsDocument - Document to 'investigate'
 * @return bool - True if plaintext, false if binary
 */
function isPlainTextFile(imsDocument)
{
    //When the command 'open <path>' is executed, macOS automatically determines
    //which application should be used to open the file.
    command = 'file --mime ' + imsDocument.filePath;
    var fileReturn = '';

    try {
        fileReturn = child_process.execSync(command);
    }
    catch(e) {
        return true;
    }

    if(fileReturn.indexOf('binary') === -1) {
        return true;
    } else {
        return false;
    }
}


/**
* ericd 011216
* The TransferAgent uses the 'lsof' utility only for files with the following extensions
* Func is used instead of if(isPlainTextFile): if(!hasKnownLsofExt) in openfile.js: openFile
*/
function hasKnownLsofExt(imsDocument)
{

	// SD09285 gv 13-04-2018 never use lsof on OSX 10.13
	// return false; now we have lsof timeout changed

	var knownExtensions =  config.getlsofExtentions(); // config.getConfigItem('lsofextentions'); //


	var str = objectToString(knownExtensions);
	logging.logDebug('hasKnownLsofExt knownExtensions str '+str);


    //var knownExtensions = ['doc', 'docx', 'docm', 'xls', 'xlsx', 'xlsm', 'ppt', 'pptx', 'pptm', 'pps', 'ppsm', 'rtf'];
    var ext = imsDocument.fileName.substr(imsDocument.fileName.lastIndexOf('.')+1);
    var fLen, i;
    fLen = knownExtensions.length;
    for (i = 0; i < fLen; i++) {
        if(ext === knownExtensions[i]) {
        	logging.logDebug('hasKnownLsofExt valid ext('+ext+') === knownExtensions[i] ('+ knownExtensions[i]+')');
            return true;
        }
    }
    return false;
}



/**
 * Śhows a MessageBox which tells the user to click 'OK' when they are done
 * editing a file. When the MessageBox is closed, the TransferAgent will shut
 * down and upload/unlock the document if necessary (changed and opend RW).
 */
function waitForMessageBox()
{
    var buttons = ['Ok'];

    var dialogOptions = {
        'type' : 'info',
        'title' : ML('Klaar met bestand', 'File is ready'),
        'message' : ML('Klik op \'OK\' als u klaar bent met het bewerken van dit bestand.',
                             'Click on \'OK\' when you are ready with editing of this file.'),
        'buttons' : buttons,
        'defaultID' : 0
    };

    dialog.showMessageBox(dialogOptions);
}


// 20170802 KVD Stack trace
function stacktrace(level)
{
    // trigger a dummy exception to enforce stack trace
    try{i.don_t.exist()} catch(e){
        var stack = e.stack.split(" at ");
        // show file name and line number
        if (level && stack.length > level) {
          var res = stack[level].replace(/^(.+?\().+?([^\/]+)$/g, " ($2");
          return res;
        }

        var output = "";
        var first = true;
        for (var el in stack) {
            if (!first) {
                var line = stack[el];
                output += line;
            }
            first = false;
        }
    }
    return '\n' + output;
}

//ericd 011216 debugging
function objectToString(object)
{
    var s = "";
    for(var propertyName in object) {

        var obj = object[propertyName];
        if (typeof(obj) == 'object') {
//            obj = '{\n' + objectToString(obj, level+1) + '\n}\n';
        }
        //s = s + "[" + propertyName +"] => "+ obj+"\r\n ";

    	s = s + "[" + propertyName +"] => ";

        try {
        	s = s + obj;
        } catch (e) {
        	s = s + e ;
        }


        s = s + "\r\n ";


    }
    return s;
}

function showDialog(string)
{
    var buttons = ['OK'];
    var dialogOptions = {
       'type' : 'info',
       'title' : 'dialog',
       'message' : string,
       'buttons' : buttons
    }
    dialog.showMessageBox(dialogOptions,
           function callback(buttonIndex) {
               if(buttonIndex === 0) {
                   //app.quit();
               }
           });
}

function ML(nl,en)
{
//return nl; // config.getConfigItem() geeft een FOUT ?!?!?!?!
    if(config.getConfigItem('language') == "en" && en) {
        return en;
    }
    return nl;
}
